import re
import numpy as np
import pandas as pd
from nltk.corpus import wordnet
from nltk.stem import WordNetLemmatizer
from nltk import word_tokenize
import nltk

nltk.download('wordnet')
nltk.download('punkt')
nltk.download('averaged_perceptron_tagger')

wordnet_lemmatizer = WordNetLemmatizer()


def __get_wordnet_pos__(treebank_tag):
    """
    return WORDNET POS compliance to WORDNET lemmatization (a,n,r,v)
    """
    if treebank_tag.startswith('J'):
        return wordnet.ADJ
    elif treebank_tag.startswith('V'):
        return wordnet.VERB
    elif treebank_tag.startswith('N'):
        return wordnet.NOUN
    elif treebank_tag.startswith('R'):
        return wordnet.ADV
    else:
        # As default pos in lemmatization is Noun
        return wordnet.NOUN


def lemmatize_string(text):
    """
    Lemmatize a single string by tokenising and then reassembling it
    :param text: the string to lemmatize
    :return: the lemmatized string
    """
    tokens = word_tokenize(text)
    pos_tokens = nltk.pos_tag(tokens)
    output = []
    for token in pos_tokens:
        output.append(wordnet_lemmatizer.lemmatize(token[0], __get_wordnet_pos__(token[1])))
    return ' '.join(output)


def lemmatize(data, columns):
    """
    Lemmatize the content of a specific column in a DataFrame
    :param data: the DataFrame
    :param columns: a single column name or list of column names
    :return: the modified DataFrame
    """
    df = data.copy()
    if type(columns) is str:
        columns = [columns]

    for column in columns:
        df[column] = df[column].apply(lambda x: lemmatize_string(x) if x is not np.nan else np.nan)
    return df


def remove_dont_knows(data, columns) -> pd.DataFrame:
    """
    Remove 'don't know' answers from a dataframe. This removes
    any rows that only contain 'don't know'. __Note__ that if multiple columns
    are supplied this will remove a row where any of the columns is 'dont know'
    :param data: the DataFrame
    :param columns: a single column name or list of column names
    :return: the modified DataFrame
    """
    df = data.copy()
    if type(columns) is str:
        columns = [columns]
    for column in columns:
        df['dontknow'] = df[column].apply(lambda x: __replace_dont_knows__(x, '') if x is not np.nan else np.nan)
        df['dontknow'].replace('', np.nan, inplace=True)
        df.dropna(subset=['dontknow'], inplace=True)
        df.drop(columns=['dontknow'], axis=1, inplace=True)
    return df


def replace_dont_knows(data, columns, replacement='') -> pd.DataFrame:
    df = data.copy()
    if type(columns) is str:
        columns = [columns]
    for column in columns:
        df[column].apply(lambda x: __replace_dont_knows__(x, replacement) if x is not np.nan else np.nan)
        df[column].replace('', np.nan, inplace=True)
    return df


def __replace_dont_knows__(text, replacement):
    """
    Replace 'dont know' or any of its synonyms in a string with the replacement string
    :param text: the string to modify
    :param replacement: the replacement string
    :return:
    """
    terms = ['i honestly dont know', "im dont know", "i m dont know", "i really dont know", "i dont really know",
             "i dont know really", "i dont know sorry", "sorry i dont know", "i dont know mate", "i dont know",
             "i don t know", "i do not know", "dont really know", "dont know", "i dunno", "dunno", "don t know", "idk",
             "do not know"]
    for word in terms:
        text = text.replace(word, replacement)

    terms = ['i am not really sure', 'im not too sure', 'im not really sure', 'not really sure', "i have no idea",
             "i am not sure", "im not sure", "i m not sure", "im unsure", "unsure", "not sure", "not too sure",
             "no idea", "not a clue", "no clue"]
    for word in terms:
        text = text.replace(word, replacement)

    terms = ["no comments", "no comment", "no opinion", "n a", "na", "none", "unknown", "no answer", 'not applicable',
             'nil', 'no']
    for word in terms:
        text = text.replace(word, replacement)

    terms = ['i cant think of anything', 'cant think of anything']
    for word in terms:
        text = text.replace(word, replacement)

    return text.strip()


def replace_all_domain_terms(text):
    """
    Replace any domain terms within a string
    TODO make this user-specified rather than hardcoded
    :param text: the string to process
    :return: the processed string with domain terms normalised
    """
    onetoone = ['1 2 1', '1 on 1', '1 to 1', '1-1', '1on1', '1:1', '1to1']
    vle = ['blackboard', 'moodle', 'canvas']
    online = ['online learning', 'learning online',
              'learn online', 'teaching online',
              'online teaching', 'online courses', 'online course',
              'online classes'
              'online class', 'online lessons', 'online lesson',
              'online lectures', 'online lecture'
              ]
    online_meeting_tool = ['zoom', 'blackboard collaborate', 'microsoft teams', 'big blue button', 'teams']

    text = replace_domain_terms(text, onetoone, 'onetoone')
    text = replace_domain_terms(text, vle, 'vle')
    text = replace_domain_terms(text, online, 'online learning')
    text = replace_domain_terms(text, online_meeting_tool, 'onlinemeetingtool')

    return text


def replace_domain_terms(text, domain_terms, replacement):
    """
    Replace domain terms within text
    :param text: the text to process
    :param domain_terms: the list of domain terms
    :param replacement: the replacement for the domain terms
    :return: the processed string
    """
    for word in domain_terms:
        text = text.replace(word, replacement)
    return text


def __clean__(df, column, inplace):

    df = df.copy()

    if not inplace:

        cleaned = 'cleaned'
        if cleaned in df.columns:
            cleaned = cleaned + "_" + column

        df[cleaned] = df[column]
        column = cleaned

    # remove apostrophes
    df[column] = df[column].apply(
        lambda x: str(x).replace("'", '').replace("`", '').replace("’", '').replace("’", '')
        if x is not np.nan else np.nan)

    # lowercase
    df[column] = df[column].apply(
        lambda x: x.lower()
        if x is not np.nan else np.nan)

    # Domain terms
    df[column] = df[column].apply(
        lambda x: replace_all_domain_terms(x)
        if x is not np.nan else np.nan)

    # remove punctuation, remove extra whitespace in string and on both sides of string
    df[column] = df[column].apply(
        lambda x: re.sub(r'[^a-z]', ' ', x).replace(' +', ' ').strip()
        if x is not np.nan else np.nan)

    df[column].replace('', np.nan, inplace=True)

    return df


def clean(data, columns, inplace=False) -> pd.DataFrame:
    """
    Cleans a dataframe
    :param data: the dataframe to clean
    :param columns: a single column name or list of column names to clean
    :param inplace: if True, changes are made to the specified column; otherwise, a 'cleaned'
    column is appended to the dataframe
    :return: the cleaned dataframe
    """
    df = data.copy()

    if type(columns) is str:
        columns = [columns]

    for column in columns:
        df = __clean__(df, column, inplace)
    return df



import pandas as pd
import qualkit.clean


def test_replace_dont_knows():
    text = qualkit.clean.__replace_dont_knows__('i dont know', 'idk')
    assert text == 'idk'


def test_replace_domain_terms():
    text = 'I use Blackboard'
    domain = ['Blackboard', 'Moodle']
    text = qualkit.clean.replace_domain_terms(text, domain, "a VLE")
    assert text == 'I use a VLE'


def test_lemmatize():
    text = {"text": ['more seminars running helping cooks find']}
    df = pd.DataFrame(text, columns=['text'])
    df = qualkit.clean.lemmatize(df, 'text')
    output = df['text'][0]
    assert output == 'more seminar run help cook find'


def test_lemmatize_string():
    output = qualkit.clean.lemmatize_string('more seminars running helping cooks find')
    assert output == 'more seminar run help cook find'


def test_remove_dont_knows():
    text = {"text":
            [
                'i think it needs more salt',
                'i really dont know',
                'i dont know',
                'i have no idea',
                'no comment',
                'its too spicy',
                'idk'
            ]
            }
    df = pd.DataFrame(text, columns=['text'])
    df = qualkit.clean.remove_dont_knows(df, 'text')
    assert df.size == 2
    assert df['text'].iloc[0] == 'i think it needs more salt'


def test_clean():
    text = {"text": ["I'm a teapot"]}
    df = pd.DataFrame(text, columns=['text'])
    df = qualkit.clean.clean(df, 'text')
    assert df['cleaned'].iloc[0] == 'im a teapot'
    assert df['text'].iloc[0] == 'I\'m a teapot'


def test_clean_inner():
    text = {"text": ["-"]}
    df = pd.DataFrame(text)
    df = qualkit.clean.__clean__(df, 'text', inplace=False)
    assert df['text'].iloc[0] == '-'
    assert pd.isnull(df['cleaned'].iloc[0])


def test_clean_inplace():
    text = {"text": ["I'm a teapot"]}
    df = pd.DataFrame(text, columns=['text'])
    df = qualkit.clean.clean(df, 'text', inplace=True)
    assert df['text'].iloc[0] == 'im a teapot'


def test_clean_multiple_inplace():
    text = {
            "text1": ["I'm a teapot", "", "row three :)"],
            "text2": ['', '', ''],
            "text3": ['Short and stout', 'Bananas are the only fruit!', '']
            }
    df = pd.DataFrame(text)
    df = qualkit.clean.clean(df, ['text1', 'text2', 'text3'], inplace=True)
    assert df['text1'].iloc[0] == 'im a teapot'


def test_clean_multiple():
    text = {
            "text1": ["I'm a teapot", "", "row three :)"],
            "text2": ['', '', ''],
            "text3": ['Short and stout', 'Bananas are the only fruit!', '']
            }
    df = pd.DataFrame(text)
    df = qualkit.clean.clean(df, ['text1', 'text2', 'text3'])
    assert df['cleaned'].iloc[0] == 'im a teapot'



import nltk

nltk.download('wordnet')
nltk.download('stopwords')

# initiate stopwords from nltk
stopwords = nltk.corpus.stopwords.words('english')

# add additional missing terms
stopwords.extend(
    ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w',
     'x', 'y', 'z', "about", "across", "after", "all", "also", "an", "and", "another", "added",
     "any", "are", "as", "at", "basically", "be", "because", 'become', "been", "before", "being", "between", "both",
     "but", "by", "came", "can", "come", "could", "did", "do", "does", "each", "else", "every", "either", "especially",
     "for", "from", "get", "given", "gets",
     'give', 'gives', "got", "goes", "had", "has", "have", "he", "her", "here", "him", "himself", "his", "how", "if",
     "in", "into", "is", "it", "its", "just", "lands", "like", "make", "making", "made", "many", "may", "me", "might",
     "more", "most", "much", "must", "my", "never", "perhaps", "no", "now", "of", "on", "only", "or", "other", "our", "out", "over", "re", "said", "same",
     "see", "should", "since", "so", "some", "still", "such", "seeing", "see", "take", "than", "that", "the", "their",
     "them", "then", "there",
     "these", "they", "this", "those", "through", "to", "too", "under", "up", "use", "using", "used", "underway",
     "very", "want", "was", "way", "we", "well", "were", "what", "when", "where", "which", "while", "whilst", "who",
     "will", "with", "would", "you", "your", "i", "i m", "im",
     'ha', 'le', 'u', 'wa',
     'etc', 'via', 'eg', 'e g', 'ie'])




import nltk
from nltk.sentiment import SentimentIntensityAnalyzer

nltk.download('vader_lexicon')


def add_sentiment_score(data, column, filter=None):
    data = data.copy()
    sia = SentimentIntensityAnalyzer()
    data['sentiment'] = data[column].apply(lambda x: sia.polarity_scores(x)['compound'])
    if filter == 'positive':
        data.drop(data.index[data['sentiment'] < 0.3], inplace=True)
    elif filter == 'negative':
        data.drop(data.index[data['sentiment'] > 0], inplace=True)
    elif filter == 'neutral':
        data.drop(data.index[data['sentiment'] > 0.3], inplace=True)
        data.drop(data.index[data['sentiment'] < 0], inplace=True)
    else:
        pass
    return data



import pandas as pd
import numpy as np
import nltk
from nltk.stem import WordNetLemmatizer
from nltk import ngrams
from rake_nltk import Rake
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.decomposition import LatentDirichletAllocation
import collections

from qualkit.stopwords import stopwords

nltk.download('wordnet')

# The code for combining LDA and RAKE is based upon Lowri Williams' method described here:
# https://github.com/LowriWilliams/Topic_Modelling_Beyond_Tokens/

# initiate stopwords
stop_words = stopwords

# initiate nltk lemmatiser
wordnet_lemmatizer = WordNetLemmatizer()


def convert_to_tokens(data):
    df = data.copy()

    # tokenise string
    df['tokens'] = df['cleaned'].apply(lambda x: nltk.word_tokenize(x))

    # remove stopwords
    df['tokens'] = df['tokens'].apply(lambda x: [item for item in x if item not in stop_words])

    # lemmatise words
    df['tokens'] = df['tokens'].apply(lambda x: [wordnet_lemmatizer.lemmatize(y) for y in x])

    # Remove nulls
    df['tokens'].replace('', np.nan, inplace=True)
    df.dropna(subset=['tokens'], inplace=True)

    return df


def create_lda_model(num_topics=10):
    # initisalise LDA Model
    lda_model = LatentDirichletAllocation(n_components=num_topics, # number of topics
                                      random_state=10,          # random state
                                      evaluate_every=-1,      # compute perplexity every n iters, default: Don't
                                      n_jobs=-1,              # Use all available CPUs
                                     )

    return lda_model


# Remove duplicates from the keywords extracted from the topic modelling output.
# This is because we want to limit the amount of Ideas that go across multiple topics.
# We want to stick to 1 idea to 1 topic
def remove_duplicates(topic_keywords):

    dupes = []

    for i in topic_keywords:
        for j in i:
            dupes.append(str(j))

    dupes = [item for item, count in collections.Counter(dupes).items() if count > 1]

    topic_keywords_processed = []

    for i in topic_keywords:
        tmp = []
        for j in i:
            if str(j) not in dupes:
                tmp.append(str(j))
            else:
                pass
        topic_keywords_processed.append(tmp)

    return topic_keywords_processed


def add_keywords_to_topics(data):
    df = data.copy()
    all_results = []

    for i in df['Dominant_topic'].unique():
        topic = df[df['Dominant_topic'] == i]
        topic = topic.copy()

        key_words = []
        topic.dropna(inplace=True)

        # run keyword extraction
        for j in topic['cleaned'].values.tolist():
            r = Rake(stopwords=stop_words)

            a = r.extract_keywords_from_text(j)
            c = r.get_ranked_phrases_with_scores()

            for k in c:
                if k not in key_words:
                    key_words.append(k)

        key_words = pd.DataFrame(key_words, columns=['score', 'term'])
        key_words = key_words.sort_values('score', ascending=False)
        key_words = key_words.drop_duplicates(subset=['term'])
        key_words['topic_number'] = i
        key_words['term_list'] =  key_words.term.apply(lambda x: x.split())

        # find bigrams from key words to match against topic modelling output
        tmp_keywords = []
        for j in key_words.values.tolist():
            tmp = []
            bi_grams = ngrams(j[3], 2)
            for g in bi_grams:
                tmp.append(' '.join(g))
            for k in j[3]:
                # lemmatise words to match the lemmatised output of the topic modelling word extraction
                tmp.append(wordnet_lemmatizer.lemmatize(k))
            j.remove(j[3])
            j.append(list(set(tmp)))
            tmp_keywords.append(j)

        # mask key words against topic modelling output
        key_words = pd.DataFrame(tmp_keywords, columns=['score', 'term', 'topic_number', 'term_list'])
        topic_keywords = topic['Topic_keywords'].values.tolist()
        topic_keywords = [item for sublist in topic_keywords for item in sublist]
        topic_keywords = list(set(topic_keywords))

        tmp = []

        for t in topic_keywords:

            mask = key_words.term_list.apply(lambda x: t in x)
            key_words_processed = key_words[mask]

            if key_words_processed.empty:
                pass
            else:
                for j in key_words_processed[['score', 'term', 'topic_number', 'term_list']].values.tolist():
                    if j not in tmp:
                        tmp.append(j)

        key_words = pd.DataFrame(tmp, columns=['score', 'term', 'topic_number', 'term_list'])

        # select the max score as the topic title
        top_key_words = key_words[key_words.score == key_words['score'].max()]

        # select the remaining keywords as child terms
        remaining_keywords = key_words[key_words.score != key_words['score'].max()]

        # if there are more than 1 keyword in the topic title, aggregate them with a / as a separatore
        top_key_words = top_key_words.copy()
        top_key_words = top_key_words.groupby(['score', 'topic_number']).agg({'term' : lambda x: ' / '.join(map(str, x))})
        top_key_words = top_key_words.reset_index()
        top_key_words['parent'] = ''

        # add 0.1 to the child keywords to identify then in the merged dataframe
        remaining_keywords = remaining_keywords.copy()
        remaining_keywords['topic_number'] = remaining_keywords['topic_number'] + 0.1
        remaining_keywords['parent'] = top_key_words['term'].values.tolist()[0]

        all_topics = pd.concat([top_key_words, remaining_keywords], sort=False)

        for t in all_topics.to_dict(orient='records'):
            all_results.append(t)

    return all_results


#
# Extracts the features from the trained model along with
# their weights
#
def lda_features(lda_model, vectorizer):
    top_feature_data = []
    for topic_idx, topic in enumerate(lda_model.components_):
        top_features_ind = topic.argsort()[:-10 - 1:-1]
        top_features = [vectorizer.get_feature_names()[i] for i in top_features_ind]
        weights = topic[top_features_ind]
        for i in range(0, topic_idx+1):
            top_feature_data.append(
                {
                    "topic": topic_idx+1,
                    "features": top_features[i],
                    "weights": weights[i]
                }
            )
    return pd.DataFrame(top_feature_data)


def lda(data, sentiment='positive', num_topics=12, output='default'):
    df = data.copy()
    df = convert_to_tokens(df)

    # Build the model
    lda_model = create_lda_model(num_topics=num_topics)

    # initialise the count vectorizer
    vectorizer = CountVectorizer(analyzer='word', ngram_range=(1, 2))

    # join the processed data to be vectorised
    vectors = []
    for index, row in df.iterrows():
        vectors.append(", ".join(row['tokens']))
    vectorised = vectorizer.fit_transform(vectors)

    # Apply model to vectorised tokens
    lda_output = lda_model.fit_transform(vectorised)

    if output == 'features':
        return lda_features(lda_model, vectorizer)

    # Add column names
    topic_names = ["Topic" + str(i) for i in range(1, lda_model.n_components + 1)]

    # make the pandas dataframe
    df_document_topic = pd.DataFrame(np.round(lda_output, 2), columns = topic_names)

    # get dominant topic for each document
    dominant_topic = (np.argmax(df_document_topic.values, axis=1)+1)
    df_document_topic['Dominant_topic'] = dominant_topic

    # join to original dataframes
    df = pd.merge(df, df_document_topic, left_index = True, right_index = True, how = 'outer')

    keywords = np.array(vectorizer.get_feature_names())

    topic_keywords = []

    for topic_weights in lda_model.components_:
        top_keyword_locs = (-topic_weights).argsort()[:20]
        topic_keywords.append(keywords.take(top_keyword_locs))

    topic_keywords = remove_duplicates(topic_keywords)

    # Topic - Keywords Dataframe
    df_topic_keywords = pd.DataFrame(topic_keywords)
    df_topic_keywords.columns = ['Term '+ str(i) for i in range(1, df_topic_keywords.shape[1] + 1)]
    df_topic_keywords['Topic_keywords'] = df_topic_keywords.values.tolist()
    df_topic_keywords['Topic_number'] = df_topic_keywords.index + 1
    df_topic_keywords = df_topic_keywords[['Topic_keywords', 'Topic_number']]

    # Remove None from lists
    tmp = []

    for i in df_topic_keywords['Topic_keywords']:
        tmp.append([x for x in i if x is not None])

    df_topic_keywords['Topic_keywords'] = tmp

    # Merge key terms back to main frame
    df = pd.merge(df, df_topic_keywords, left_on='Dominant_topic', right_on='Topic_number')

    return df


def lda_with_keywords(data):
    df = data.copy()
    # Add keywords to topics
    all_results = add_keywords_to_topics(df)

    all_topics_df = pd.DataFrame(all_results)
    all_topics_df = all_topics_df.sort_values('topic_number', ascending=True)

    return all_topics_df


import pandas as pd
import numpy as np
from sklearn.feature_extraction.text import TfidfVectorizer
from corextopic import corextopic as ct
from qualkit.stopwords import stopwords


def load_topics(file):
    """
    Loads topic names and anchors from a file
    :param file: the path/filename of the file to use
    :return: a DataFrame with 'topic_name' and 'anchors' columns
    """
    df = pd.read_csv(file, header=None)
    df['anchors'] = df.drop(0, axis=1).values.tolist()
    df['anchors'] = df['anchors'].apply(lambda x: [i for i in x if i == i])
    df['topic_name'] = df[0]
    output = df[['topic_name', 'anchors']].copy()
    return output


def anchored_topic_model(data, column, topic_filename=None, topic_names=None, anchors=None, number_of_topics=10,
                         print_topic_details=False):
    """
    Creates a topic model using the Corex algorithm using an optional set of user-provided anchors
    :param data: a DataFrame containing a column with text to analyse
    :param column: the name of the column containing the text
    :param topic_filename: (optional) the name of a file containing anchor terms and topic names
    :param topic_names: (optional) a List of topic names
    :param anchors: (optional) a list containing lists of anchor terms
    :param number_of_topics: defaults to 10; the number of topics to generate (overridden if topics are supplied)
    :param print_topic_details: if true, print to console a summary of the generated topics
    :return:a DataFrame containing the original data supplied along with the 'topic label' and 'topic name' for each row
    """

    # Load topics from file if provided
    if topic_filename is not None:
        tf = load_topics(topic_filename)
        topic_names = tf['topic_name']
        anchors = tf['anchors']

    df = data.copy()

    # Calculate number of topics based on provided anchors if supplied
    if anchors is not None:
        number_of_topics = len(anchors)
        if topic_names is not None:
            if len(topic_names) != len(anchors):
                print("Topic names and anchors must be the same length")
                return None
            topic_names = ['No matching topic']+topic_names

    # Initialise the vectorizer that will split the text into tokens
    # uses the TFIDF algorithm
    vectorizer = TfidfVectorizer(
        max_df=0.25,
        min_df=5,
        max_features=None,
        ngram_range=(1, 3),
        norm=None,
        binary=True,
        use_idf=False,
        sublinear_tf=False,
        stop_words=stopwords
    )

    # Get rid of any NaNs before we run the vectorizer
    df.dropna(subset=[column], inplace=True)

    vectorizer = vectorizer.fit(df[column])
    tfidf = vectorizer.transform(df[column])
    vocab = vectorizer.get_feature_names()

    # Filter by vocab
    if anchors is not None:
        anchors = [
            [a for a in topic if a in vocab]
            for topic in anchors
        ]

    # Create the Corex model
    model = ct.Corex(n_hidden=number_of_topics, seed=42)
    if anchors is None:
        model = model.fit(
            tfidf,
            words=vocab
        )
    else:
        model = model.fit(
            tfidf,
            words=vocab,
            anchors=anchors,
            anchor_strength=2  # Tells the model how much it should rely on the anchors
        )

    # Enumerate the topics from the model and create labels
    topic_labels = ['No matching topic']
    if print_topic_details:
        print("Total correlation: " + str(model.tc))
    for i, topic_ngrams in enumerate(model.get_topics(n_words=10)):
        topic_ngrams = [ngram[0] for ngram in topic_ngrams if ngram[1] > 0]
        topic_label = ", ".join(topic_ngrams)
        topic_labels.append(topic_label)
        if print_topic_details:
            print("Topic #{}: {}".format(i + 1, topic_label))
    if topic_names is None:
        topic_names = topic_labels

    # Apply the model to the documents
    doc_topics = model.transform(tfidf)
    topic_matrix = pd.DataFrame(
        doc_topics,
        columns=["topic_{}".format(i) for i in range(1, number_of_topics + 1)],
        index=df.index,
    )

    # Add columns for each topic
    topic_cols = ["topic_" + str(i) for i in range(1, number_of_topics + 1)]
    df_document_topic = pd.DataFrame(topic_matrix, columns=topic_cols)
    df_document_topic.insert(0, 'no topic', False, True)

    # get the dominant topic for each document, or 'no topic' if unmatched
    dominant_topic = (np.argmax(df_document_topic.values, axis=1))
    df_document_topic['Dominant_topic'] = dominant_topic
    df_document_topic['Topic label'] = df_document_topic['Dominant_topic'].apply(lambda x: topic_labels[x])
    df_document_topic['Topic name'] = df_document_topic['Dominant_topic'].apply(lambda x: topic_names[x])

    # remove the individual columns for topics
    df_document_topic.drop(columns=topic_cols, inplace=True, axis=1)
    topic_matrix.drop(columns=topic_cols, inplace=True, axis=1)

    # join the results back to the original dataframe and return
    topic_matrix = pd.merge(topic_matrix, df_document_topic[['Topic label', 'Topic name']],
                            left_index=True, right_index=True, how='outer')
    results = pd.merge(df, topic_matrix, left_index=True, right_index=True, how='outer')

    return results



from rake_nltk import Rake
import nltk
from nltk.stem import WordNetLemmatizer

from qualkit.stopwords import stopwords

nltk.download('wordnet')

# initiate nltk lemmatiser
lemma = WordNetLemmatizer()

r = Rake(stopwords=stopwords, min_length=1, max_length=4)


def rake_implement(x, r):
    r.extract_keywords_from_text(x)
    return r.get_ranked_phrases()


def add_keywords(df, column):
    df = df.copy()
    df['keywords'] = df[column].apply(lambda x: rake_implement(x, r))
    df['keywords'] = df['keywords'].apply(lambda x: [lemma.lemmatize(y) for y in x])
    df = df.explode('keywords')
    df.dropna(subset=['keywords'], inplace=True)
    return df




from setuptools import setup

VERSION = '0.1.2'

setup(
    name='qualkit',
    version=VERSION,
    packages=['qualkit'],
    url='https://github.com/JiscDACT/qualkit',
    download_url='https://github.com/JiscDACT/qualkit/tarball/{}'.format(VERSION),
    license='BSD',
    author='Scott Wilson',
    author_email='scott.wilson@jisc.ac.uk',
    description='Python qualitative analysis toolkit with utilities and simplified wrappers for common algorithms',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    install_requires=['pandas>=1.1.5',
                        'numpy>=1.19.5',
                        'nltk>=3.6.2',
                        'scikit-learn>=0.24.2',
                        'corextopic>=1.1'],
    entry_points={
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: BSD License',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ]
)


from qualkit import anchored_topic_model


def test_load_topics():
    df = anchored_topic_model.load_topics('test/topics.csv')
    assert df['topic_name'].size == 3
    assert df['anchors'].loc[0] == ['apple', 'banana']
    assert df['anchors'].loc[2] == ['potato', 'carrot', 'onion']